(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.1'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      7286,        229]*)
(*NotebookOutlinePosition[      7930,        251]*)
(*  CellTagsIndexPosition[      7886,        247]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Writing  a Link \"Spider\" Program Using .NET/Link", "Title"],

Cell["\<\
In this exercise you will work to create a simple spider program that \
collects all the links from a web page and then follows those links to \
accumulate links on those pages, etc. The program will return a list of all \
the http URLs it finds. Please limit your search to a depth of 2, as the \
number of links to visit grows very quickly with depth, and you want your \
program to run in a relatively short amount of time.\
\>", "Text"],

Cell["\<\
The starting point is to create an instance of Internet Explorer, which \
exposes a COM interface for scripting. .NET/Link is just as comfortable \
calling COM objects as native .NET objects. There are other possible ways of \
writing this program. For example, you could use native .NET classes to read \
HTML from a URL and then parse it yourself to extract the <a> tags, or use \
XML-related functionality, but that might run into trouble if the HTML is not \
well-formed XML (and many pages are not). It's quite easy to just use the \
Internet Explorer object as the starting point and use its Document Object \
Model.\
\>", "Text"],

Cell["\<\
<<NETLink`;
InstallNET[];\
\>", "Input"],

Cell["\<\
Use the CreateCOMObject function to create a new instance of a COM object:\
\>", "Text"],

Cell[CellGroupData[{

Cell["ie = CreateCOMObject[\"InternetExplorer.Application\"]", "Input"],

Cell[BoxData[
    InterpretationBox[\(\[LeftGuillemet]NETObject[
          COMInterface["SHDocVw.IWebBrowser2"]] \[RightGuillemet]\),
      NETLink`Objects`NETObject$16336286$134217729]], "Output"]
}, Open  ]],

Cell["\<\
You can tell by the COMInterface[...] in the above object that it is just a \
.NET wrapper around a COM object, not a true .NET object.\
\>", "Text"],

Cell["\<\
Use the navigate() method to go to a URL. You must wait until the page is \
loaded to call any methods that inspect the content.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(ie@navigate["\<www.wolfram.com\>"];\)\), "\[IndentingNewLine]", 
    \(\(While[ie@busy, \ Pause[0.5]];\)\), "\[IndentingNewLine]", 
    \(doc\  = \ ie@Document\)}], "Input"],

Cell[BoxData[
    InterpretationBox[\(\[LeftGuillemet]NETObject[
          "mshtml.HTMLDocumentClass"] \[RightGuillemet]\),
      NETLink`Objects`NETObject$167772161]], "Output"]
}, Open  ]],

Cell["\<\
Although it is not necessary, you can make the IE window visible if you want:\
\
\>", "Text"],

Cell["ie@Visible = True;", "Input"],

Cell["The links property gives a collection of all the links:", "Text"],

Cell[CellGroupData[{

Cell["links = doc@links", "Input"],

Cell[BoxData[
    InterpretationBox[\(\[LeftGuillemet]NETObject[
          "mshtml.IHTMLElementCollection"] \[RightGuillemet]\),
      NETLink`Objects`NETObject$538579828$184549377]], "Output"]
}, Open  ]],

Cell["links@length", "Input"],

Cell["\<\
The item() method retrieves an element from the collection (0-based index):\
\>", "Text"],

Cell[CellGroupData[{

Cell["links@item[0]", "Input"],

Cell[BoxData[
    InterpretationBox[\(\[LeftGuillemet]NETObject[
          "mshtml.HTMLAnchorElementClass"] \[RightGuillemet]\),
      NETLink`Objects`NETObject$218103809]], "Output"]
}, Open  ]],

Cell["\<\
Use the toString() method to get the URL of the link as a string:\
\>", "Text"],

Cell[CellGroupData[{

Cell["links@item[0]@toString[]", "Input"],

Cell[BoxData[
    \("http://www.wolfram.com/"\)], "Output"]
}, Open  ]],

Cell["\<\
That's all you need to write a LinkSpider program. Remember that you can use \
the NETTypeInfo function to get information about the methods and properties \
that each object supports. Be aware, however, that for .NET types that are \
wrappers of COM objects, there tend to be a ridiculously large number of \
bizarrely-named methods and properties. Part of this is because COM is so \
messy and part is because of the way that COM objects are exposed to .NET.

Execute if you dare (normal .NET objects have clean interfaces that look \
nothing like this!):\
\>", "Text"],

Cell["\<\
NETTypeInfo[links@item[0], {\"Methods\", \"Properties\"}, Inherited->False]\
\>", "Input"],

Cell[CellGroupData[{

Cell["\<\
A simple LinkSpider program (don't peek until you've tried your own)\
\>", "Subsubsection"],

Cell["\<\
LinkSpider[url_String, depth_Integer:2] :=
\tNETBlock[
\t\tInstallNET[];
\t\tie = CreateCOMObject[\"InternetExplorer.Application\"];
\t\t(* ie@Visible = True; *)
\t\tgetLinks[ie, url, {}, depth]
\t]

getLinks[ie_?NETObjectQ, url_String, accum_List, depth_Integer] :=
\tModule[{doc, linkCollection, i, linksOnPage, subLinks},
\t\tie@navigate[url];
\t\tWhile[ie@busy, Pause[0.5]];
\t\tdoc = ie@Document;
\t\tlinkCollection = doc@links;
\t\tlinksOnPage = {};
\t\tFor[i = 0, i < linkCollection@length, i++,
\t\t\tthisURL = linkCollection@item[i]@toString[];
\t\t\tIf[StringMatchQ[thisURL, \"http:*\"],
\t\t\t\tAppendTo[linksOnPage, thisURL]
\t\t\t]
\t\t];
\t\tJoin[accum, linksOnPage, If[depth > 1, Flatten[getLinks[ie, #, accum, \
depth - 1]& /@ linksOnPage], {}]]
\t]\
\>", "Input"],

Cell["All the HTTP links referred to on the WRI home page:", "Text"],

Cell["LinkSpider[\"http://www.wolfram.com\", 1]", "Input"],

Cell["Down to a depth of 2 (takes more than half a minute):", "Text"],

Cell["LinkSpider[\"http://www.wolfram.com\", 2]", "Input"],

Cell["\<\
For fun, uncomment the ie@Visible=True line in the program and watch as the \
browser window visits each linked page. \
\>", "Text"]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"5.1 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 695}},
WindowSize->{908, 656},
WindowMargins->{{17, Automatic}, {4, Automatic}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 67, 0, 95, "Title"],
Cell[1846, 55, 450, 7, 71, "Text"],
Cell[2299, 64, 646, 10, 90, "Text"],
Cell[2948, 76, 50, 3, 48, "Input"],
Cell[3001, 81, 98, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[3124, 87, 71, 0, 30, "Input"],
Cell[3198, 89, 197, 3, 29, "Output"]
}, Open  ]],
Cell[3410, 95, 159, 3, 33, "Text"],
Cell[3572, 100, 152, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[3749, 107, 196, 3, 70, "Input"],
Cell[3948, 112, 178, 3, 29, "Output"]
}, Open  ]],
Cell[4141, 118, 103, 3, 33, "Text"],
Cell[4247, 123, 35, 0, 30, "Input"],
Cell[4285, 125, 71, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[4381, 129, 34, 0, 30, "Input"],
Cell[4418, 131, 193, 3, 29, "Output"]
}, Open  ]],
Cell[4626, 137, 29, 0, 30, "Input"],
Cell[4658, 139, 99, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[4782, 145, 30, 0, 30, "Input"],
Cell[4815, 147, 183, 3, 29, "Output"]
}, Open  ]],
Cell[5013, 153, 89, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[5127, 159, 41, 0, 30, "Input"],
Cell[5171, 161, 59, 1, 29, "Output"]
}, Open  ]],
Cell[5245, 165, 581, 10, 109, "Text"],
Cell[5829, 177, 100, 2, 30, "Input"],

Cell[CellGroupData[{
Cell[5954, 183, 101, 2, 29, "Subsubsection"],
Cell[6058, 187, 790, 25, 426, "Input"],
Cell[6851, 214, 68, 0, 33, "Text"],
Cell[6922, 216, 58, 0, 30, "Input"],
Cell[6983, 218, 69, 0, 33, "Text"],
Cell[7055, 220, 58, 0, 30, "Input"],
Cell[7116, 222, 142, 3, 30, "Text"]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

